// $Id: FreeEnergyModel.h 865 2015-04-21 22:41:49Z ge $
/// \file FreeEnergyModel.h
/// \brief contains the FreeEnergyModel class
///
/// $Revision: 865 $
/// \author Gerald Weber <gweberbh@gmail.com>
#ifndef GBC_FREEENERGYMODEL_H
#define GBC_FREEENERGYMODEL_H "$Id: FreeEnergyModel.h 865 2015-04-21 22:41:49Z ge $"
#include "SequenceInfo.h"
#include "SequenceDataset.h"
#include "IndividualNearestNeighbours.h"

namespace gbc
{
template<class _Model=IndividualNearestNeighbours<double> >
class FreeEnergyModel
  {
  public:
    typedef _Model                                 model_type;
    typedef typename model_type::value_type        value_type;
    typedef std::valarray<value_type>              vector_type;
    typedef typename model_type::sequenceinfo_type sequenceinfo_type;
    typedef SequenceDataset<sequenceinfo_type>     sequence_dataset_type;
    typedef std::deque<std::string>                string_deque_type;

    sequence_dataset_type dataset;
    model_type            *model;
    std::string           data_set_type;

    FreeEnergyModel(void) {}


   inline void verify(std::ostream& out)
      {
      dataset.verify(out);
      }

   inline void print_tex(std::ostream& out)
     {
     dataset.print_tex(out);
     }

    /// \brief Calculates the difference between the predicted and the adjusted temperature
    inline void predict_all(void)
      {
      typename sequence_dataset_type::sequence_info_deque_type::iterator si=dataset.Raw_dataset.begin();
      for(si=dataset.Raw_dataset.begin(); si != dataset.Raw_dataset.end(); si++)
        {
        model->Sequence_info=&(*si);
        model->reset_calculation();
        model->melting_temperature();
        }
      }
      
    //Rules are read from parameter files, make sure that this is set before read_xml
    //otherwise the rules are not applied to the dataset
    void read_rules_and_parameters_from_list_of_files(string_deque_type &par_file_deque)
      {
      dataset.read_rules_from_list_of_files(par_file_deque);
      model->get_parameters_from_list_of_files(par_file_deque);
      }


  };

}
#endif
